﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.NK.WeightInitializers {

  [Item("LinearWeightsInitializer", "Assigns linear increasing weights.")]
  [StorableClass]
  public class LinearWeightsInitializer : NamedItem, IWeightsInitializer {

    public override bool CanChangeName { get { return false; } }
    public override bool CanChangeDescription { get { return false; } }

    [StorableConstructor]
    protected LinearWeightsInitializer(bool deserializing) : base(deserializing) { }
    protected LinearWeightsInitializer(LinearWeightsInitializer original, Cloner cloner)
      : base(original, cloner) {
    }
    public LinearWeightsInitializer() {
      name = ItemName;
      description = ItemDescription;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new LinearWeightsInitializer(this, cloner);
    }

    #region IWeightsInitializer Members

    public IEnumerable<double> GetWeights(int nComponents) {
      for (int i = 0; i < nComponents; i++)
        yield return i;
    }

    #endregion
  }

}
